# coding: utf-8

import logging

from django.conf import settings
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.template import Context, Template

from .models import EmailTemplate


log = logging.getLogger('django_templated_email')


class NoTemplateError(Exception):
    pass


def _render_from_text(text, **context):
    return Template(text).render(Context(context))


def build_templated_mail(
    template_name,
    context,
    recipients,
    sender=None,
    bcc=None,
    cc=None,
    files=None,
):
    html = text = None
    message_cls = EmailMessage
    t = EmailTemplate.objects.filter(template_name__iexact=template_name).first()
    if not t:
        raise NoTemplateError('template "{}" not found'.format(template_name))
    subject = _render_from_text(t.subject, **context)
    text = _render_from_text(t.plain_text, **context)
    if t.html:
        message_cls = EmailMultiAlternatives
        html = _render_from_text(t.html, **context)

    msg = message_cls(subject, text, sender, list(recipients), bcc=bcc, cc=cc)
    if html:
        msg.attach_alternative(html, "text/html")
    files = [] if files is None else files
    for file_data in files:
        msg.attach(*file_data)
    return msg


def send_templated_mail(
    template_name,
    context,
    recipients,
    sender=None,
    bcc=None,
    cc=None,
    files=None,
    fail_silently=False
):
    # remove duplicates from bcc
    if not bcc:
        bcc = []
    if cc:
        bcc = set(bcc) - set(cc)
    bcc = list(set(bcc) - set(recipients))

    log.info('sending mail {} to {}, bcc: {}'.format(template_name, recipients, bcc))
    mail = build_templated_mail(
        template_name,
        context,
        recipients=recipients,
        sender=sender,
        bcc=bcc,
        cc=cc,
        files=files,
    )
    mail.send(fail_silently=fail_silently)



# coding: utf-8

from django.contrib import admin
from .models import EmailTemplate


class EmailTemplateAdmin(admin.ModelAdmin):
    actions = ['duplicate']
    list_display = ('template_name', 'subject',)

    def duplicate(self, request, queryset):
        for t in queryset:
            t.pk = None
            t.save()


admin.site.register(EmailTemplate, EmailTemplateAdmin)



# coding: utf-8

import os
from setuptools import setup


# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='dj-templated-mail',
    version='0.1.2',
    packages=['dj_templated_mail'],
    include_package_data=True,
    license='MIT License',
    description='Django app that provides possibility to render email messages from templates stored in DB.',
    url='https://github.com/oeegor/dj-templated-mail#django-templated-mail/',
    author='Egor Orlov',
    author_email='oeegor@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License', # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# coding: utf-8

from django.db import models


class EmailTemplate(models.Model):

    """
    Since these are more likely to be changed than other templates, we store
    them in the database.

    This means that an admin can change email templates without having to have
    access to the filesystem.
    """

    template_name = models.CharField(
        max_length=100,
    )

    subject = models.TextField()

    plain_text = models.TextField(null=True, blank=True)

    html = models.TextField(null=True, blank=True)

    def __unicode__(self):
        return u'{}'.format(self.template_name)
